package demo.linksprite.mybluetoothwatch;


import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.util.IllegalFormatCodePointException;
import java.util.concurrent.atomic.AtomicBoolean;

import demo.linksprite.mybluetoothwatch.ble.BaseActivity;
import demo.linksprite.mybluetoothwatch.tools.SpUtils;
import demo.linksprite.mybluetoothwatch.tools.StrBinaryTurn;
import demo.linksprite.mybluetoothwatch.tools.cmd;

public class MainActivity extends BaseActivity {

    private int count = 0;
    private final static String TAG = MainActivity.class.getName();
    private boolean isConnect = false;
    private BluetoothAdapter mAdapter;
    private boolean isHigh = true;
    private boolean isWatchWork = false;
    private Button btn_start, btn_stop, btn_set;
    private boolean isStop = false;
    private TextView tv_tem;
    private int MAXtemp = 30;
    private EditText edit_temp;
    private AtomicBoolean isFirst = new AtomicBoolean(true);

    private static final String TEMP = "temp";

    @Override
    public void serviceReady() {
        mService.startScan(true);
    }

    @Override
    public void onCreateBeforeDo(Bundle savedInstanceState) {

    }

    private Handler.Callback callback = new Handler.Callback() {
        @Override
        public boolean handleMessage(Message msg) {
            switch (msg.what) {
                case 1:
                    mService.send(cmd.getPhoneAlarmCmd());
                    break;
                case 2:
                    isHigh = false;
                    break;
                case 3:
                    HandRingVibration();
                    break;
                case 4:
                    if (mAdapter != null)
                        mAdapter.stopLeScan(leScanCallback);
                    break;
            }
            return false;
        }
    };

    @Override
    public Handler.Callback getCallBack() {
        return callback;
    }

    @Override
    public void initView() {
        setContentView(R.layout.activity_main);
        btn_start = (Button) findViewById(R.id.btn_start);
        btn_stop = (Button) findViewById(R.id.btn_stop);
        tv_tem = (TextView) findViewById(R.id.tv_tem);
        edit_temp = (EditText) findViewById(R.id.edit_temp);
        btn_set = (Button) findViewById(R.id.btn_set);
        btn_set.setOnClickListener(listener);
        btn_start.setOnClickListener(listener);
        btn_stop.setOnClickListener(listener);
        MAXtemp = SpUtils.getInt(this,TEMP);
        setBtnEnable(false);
    }

    private void setBtnEnable(final boolean is) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                btn_start.setEnabled(is);
                btn_stop.setEnabled(is);
            }
        });

    }

    private View.OnClickListener listener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            switch (v.getId()) {
                case R.id.btn_start:
                    isHigh = false;
                    isStop = false;
                    isFirst.getAndSet(true);
                    break;
                case R.id.btn_stop:
                    handler.removeMessages(3);
                    isHigh = true;
                    isStop = true;
                    break;
                case R.id.btn_set:
                    isHigh = false;
                    isStop = false;
                    String temp = edit_temp.getText().toString().trim();
                    MAXtemp = Integer.parseInt(temp);
                    SpUtils.setData(MainActivity.this, TEMP, MAXtemp);
                    isFirst.getAndSet(true);
                    break;
            }
        }
    };

    @Override
    public void onDeviceConnected() {

    }


    @Override
    public void onDeviceReady() {
        Log.e(TAG, "onDeviceReady");
        mAdapter = BluetoothAdapter.getDefaultAdapter();
        setBtnEnable(true);
        if (mAdapter != null)
            mAdapter.startLeScan(leScanCallback);
    }

    private BluetoothAdapter.LeScanCallback leScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {
            if (device.getName() != null && device.getName().startsWith("Linkab") && !isWatchWork)
                checkIsHigh(scanRecord);
        }
    };

    private void checkIsHigh(byte[] scanRecord) {
        String s = StrBinaryTurn.bytestoHexString(extractBytes(scanRecord, 21, 8));
        String data = s.substring(s.length() - 2);
        Log.e(TAG, "" + data + "   " + s);
        final int i = Integer.parseInt(data, 16);
        if (i >= MAXtemp && isFirst.compareAndSet(true, false)) {
            Log.e(TAG, "温度达到预定值");
        }

        if (!isFirst.get() && i < MAXtemp && i != 0 && !isHigh && !isStop) {
            isHigh = true;
            handler.sendEmptyMessage(3);
        }
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                tv_tem.setText(i + "℃");
            }
        });
//        isHigh = true;
    }

    private byte[] extractBytes(byte[] scanRecord, int start, int length) {
        byte[] bytes = new byte[length];
        System.arraycopy(scanRecord, start, bytes, 0, length);
        return bytes;
    }

    public void HandRingVibration() {
        handler.sendEmptyMessage(1);
    }

    @Override
    public void onDeviceDisconnecting() {

    }

    @Override
    public void onDeviceDisconnected() {

    }

    @Override
    public void onLinklossOccur() {

    }

    @Override
    public void onDeviceFound(BluetoothDevice device, int rssi, byte[] scanRecord) {
        Log.e(TAG, "device name:" + device.getName() + "  address:" + device.getAddress() + " rssi:" + rssi);
        if (device.getName() != null && device.getName().startsWith("GUODOO") && !isConnect) {
            isConnect = true;
            mService.startScan(false);
            mService.connectDev(device.getAddress());
        }
    }


    @Override
    public void onDiscoverServer() {

    }

    @Override
    public void onTimeOut(int staus) {

    }

    @Override
    public void BluetoothOpened() {

    }

    @Override
    public void onDataSendOver() {
        isHigh = false;
    }

    @Override
    public void onReadData(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
        super.onReadData(gatt, characteristic);
        handler.removeCallbacksAndMessages(null);
        isHigh = false;
        Log.e(TAG, "read a data:" + StrBinaryTurn.getBinaryStrFromByteArr(characteristic.getValue()));
    }
}
