package demo.linksprite.mybluetoothwatch.ble;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Binder;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Message;
import android.text.TextUtils;
import android.util.ArrayMap;
import android.util.Log;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.UUID;

import demo.linksprite.mybluetoothwatch.tools.StrBinaryTurn;
import no.nordicsemi.android.nrftoolbox.error.GattError;
import no.nordicsemi.android.nrftoolbox.utility.DebugLogger;

/**
 * Created by zsq on 2016/1/19.
 * Ble service
 */
public class BleBaseService extends Service {

    private final static String TAG = BleBaseService.class.getName();
    private Queue<Request> mQueue;
    private boolean mInitInProgress;
    private BleCallBack mBleCallBack;
    private HandlerThread handlerThread;
    public Handler mHandler;
    private BluetoothGatt mBluetoothGatt;
    private BluetoothAdapter mBluetoothAdapter;
    private String mAddress;
    private boolean mUserDisconnected = false;
    private final static int SCANTINEOUT = 10000;
    private final static int TIMEOUT = 1;

    private int mSatuts = 0;

    private final static UUID GENERIC_ATTRIBUTE_SERVICE = UUID.fromString("00001801-0000-1000-8000-00805f9b34fb");
    private final static UUID SERVICE_CHANGED_CHARACTERISTIC = UUID.fromString("00002A05-0000-1000-8000-00805f9b34fb");
    public static final UUID BLE_UUID_BATTERY_SERVICE = UUID.fromString("0000180F-0000-1000-8000-00805f9b34fb");
    public static final UUID BLE_UUID_BATTERY_LEVEL_CHAR = UUID.fromString("00002A19-0000-1000-8000-00805f9b34fb");
    public static final UUID CCCD = UUID.fromString("00002902-0000-1000-8000-00805f9b34fb");
    public static final UUID BLEDEVSOFT_REVISON_UUID = UUID.fromString("00002a26-0000-1000-8000-00805f9b34fb");
    public static final UUID BLEDEVFIRMWARE_NAME_UUID = UUID.fromString("00002a27-0000-1000-8000-00805f9b34fb");
    public static final UUID DIS_UUID = UUID.fromString("0000180a-0000-1000-8000-00805f9b34fb");
    public static final UUID RX_SERVICE_UUID = UUID.fromString("00000af0-0000-1000-8000-00805f9b34fb");
    public static final UUID RX_CHAR_UUID = UUID.fromString("00000af7-0000-1000-8000-00805f9b34fb");
    public static final UUID TX_CHAR_UUID = UUID.fromString("00000af6-0000-1000-8000-00805f9b34fb");


    private final static String ERROR_CONNECTION_STATE_CHANGE = "Error on connection state change";
    private final static String ERROR_DISCOVERY_SERVICE = "Error on discovering services";
    private final static String ERROR_AUTH_ERROR_WHILE_BONDED = "Phone has lost bonding information";
    private final static String ERROR_WRITE_DESCRIPTOR = "Error on writing descriptor";
    private final static String ERROR_READ_CHARACTERISTIC = "Error on reading characteristic";
    private final static String ERROR_NOT_SUPPORT_BLUETOOTH = "Error on support bluetooth";

    public boolean initBleService() {
        if (handlerThread == null) {
            handlerThread = new HandlerThread("linksprite.ble.baseService");
            handlerThread.start();
        }
        if (mHandler == null)
            mHandler = new Handler(handlerThread.getLooper(), mCallback);

        return init_ble();
    }


    public boolean init_ble() {
        if (mBluetoothAdapter == null) {
            if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
                mBleCallBack.onError(Constants.ERROR_NOTSUPPORTBLUETOOTH, "init_ble:" + ERROR_NOT_SUPPORT_BLUETOOTH);
                return false;
            }

            // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
            // BluetoothUUIDAdapter through BluetoothManager.
            final BluetoothManager bluetoothManager =
                    (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
            mBluetoothAdapter = bluetoothManager.getAdapter();

            // Checks if Bluetooth is supported on the device.
            if (mBluetoothAdapter == null) {
                mBleCallBack.onError(Constants.ERROR_NOTSUPPORTBLUETOOTH, "init_ble:" + ERROR_NOT_SUPPORT_BLUETOOTH);
                return false;
            }
            if (!mBluetoothAdapter.isEnabled()) {
                mBluetoothAdapter.enable();
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        return true;
    }

    private boolean isOpenBluetooth() {
        if (mBluetoothAdapter.isEnabled())
            return true;
        mBleCallBack.disOpenBluetooth();
        return false;
    }


    public void startScan(boolean isScan) {
        if (mBluetoothAdapter == null) {
            init_ble();
        }
        if (isScan) {
            mHandler.postDelayed(mScanTime, SCANTINEOUT);
            mBluetoothAdapter.startLeScan(mLeScanCallback);
        } else {
            mHandler.removeCallbacks(mScanTime);
            Log.e(TAG,"stop scan");
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
    }

    private Runnable mScanTime = new Runnable() {
        @Override
        public void run() {
            mBleCallBack.onDeviceNotFound();
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
    };

    private BluetoothAdapter.LeScanCallback mLeScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {

            String name = device.getName();
            mBleCallBack.onDeviceFound(device, rssi, scanRecord);
//            Log.e(TAG,StrBinaryTurn.bytestoHexString(scanRecord));
        }
    };


    public boolean connectDev(String addr) {
        if (initBleService() && !TextUtils.isEmpty(addr)) {
            if (mBluetoothGatt != null) {
                mBluetoothGatt.close();
            }
            final BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(addr);
            if (device != null) {
                if (device.getBondState() == BluetoothDevice.BOND_BONDED) {
                    removeBond(device);
                }
                mHandler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        mHandler.sendMessageDelayed(mHandler.obtainMessage(TIMEOUT, Constants.CONNECTTED, 0), SCANTINEOUT);
                        mBluetoothGatt = device.connectGatt(BleBaseService.this, false, mBluetoothGattCallback);
                        Log.e(TAG, "connecting " + device.getAddress());
                        if (mBluetoothGatt != null) {
                            mSatuts = Constants.CONNECTTING;
                        }
                    }
                },1000);
            }
        }
        return false;
    }

    private boolean removeBond(BluetoothDevice paramBluetoothDevice) {
        Boolean localBoolean = Boolean.valueOf(false);
        DebugLogger.d(TAG, "removeBond");
        try {
            localBoolean = (Boolean) BluetoothDevice.class.getMethod("removeBond", new Class[0]).invoke(paramBluetoothDevice, new Object[0]);
            DebugLogger.d(TAG, "removeBond ret:" + localBoolean);
            return localBoolean.booleanValue();
        } catch (NoSuchMethodException localNoSuchMethodException) {
            while (true)
                localNoSuchMethodException.printStackTrace();
        } catch (IllegalAccessException localIllegalAccessException) {
            while (true)
                localIllegalAccessException.printStackTrace();
        } catch (IllegalArgumentException localIllegalArgumentException) {
            while (true)
                localIllegalArgumentException.printStackTrace();
        } catch (InvocationTargetException localInvocationTargetException) {
            while (true)
                localInvocationTargetException.printStackTrace();
        }
    }

    private boolean refreshGatt(BluetoothGatt paramBluetoothGatt) {
        Boolean localBoolean = Boolean.valueOf(false);
        DebugLogger.d(TAG, "refreshGatt");
        try {
            localBoolean = (Boolean) BluetoothGatt.class.getMethod("refresh", new Class[0]).invoke(paramBluetoothGatt, new Object[0]);
            DebugLogger.d(TAG, "refresh ret:" + localBoolean);
            return localBoolean.booleanValue();
        } catch (NoSuchMethodException localNoSuchMethodException) {
            while (true)
                localNoSuchMethodException.printStackTrace();
        } catch (IllegalAccessException localIllegalAccessException) {
            while (true)
                localIllegalAccessException.printStackTrace();
        } catch (IllegalArgumentException localIllegalArgumentException) {
            while (true)
                localIllegalArgumentException.printStackTrace();
        } catch (InvocationTargetException localInvocationTargetException) {
            while (true)
                localInvocationTargetException.printStackTrace();
        }
    }

    private LocalBind localBind = new LocalBind();

    public class LocalBind extends Binder {
        public BleBaseService getService() {
            return BleBaseService.this;
        }
    }

    @Override
    public IBinder onBind(Intent intent) {
        return localBind;
    }

    public void setBleCallBack(BleCallBack mBleCallBack) {
        this.mBleCallBack = mBleCallBack;
    }

    public boolean disconnect() {
        mUserDisconnected = true;
        if (mSatuts != Constants.DISCONNECT && mBluetoothGatt != null) {
            mBleCallBack.onDeviceDisconnecting();
            mBluetoothGatt.disconnect();
            return true;
        }
        return false;
    }

    /**
     * Closes and releases resources. May be also used to unregister broadcast listeners.
     */
    public void close() {
        if (mBluetoothGatt != null) {
            refreshGatt(mBluetoothGatt);
            mBluetoothGatt.close();
            mBluetoothGatt = null;
        }
        mUserDisconnected = false;
    }


    private Handler.Callback mCallback = new Handler.Callback() {
        @Override
        public boolean handleMessage(Message msg) {
            switch (msg.what) {
                case TIMEOUT:
                    mBleCallBack.onTimeOut(msg.arg1);
                    break;
            }

            return false;
        }
    };


    public void onReadData(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        if (status == BluetoothGatt.GATT_SUCCESS) {
            mBleCallBack.onReadData(gatt, characteristic);
        }
        mBleCallBack.onError(Constants.ERROR_READCharacteristic, "onReadData:" + GattError.parse(status));
    }


    /**
     * When the device is bonded and has the Generic Attribute service and the Service Changed characteristic this method enables indications on this characteristic.
     * In case one of the requirements is not fulfilled this method returns <code>false</code>.
     *
     * @param gatt the gatt device with services discovered
     * @return <code>true</code> when the request has been sent, <code>false</code> when the device is not bonded, does not have the Generic Attribute service, the GA service does not have
     * the Service Changed characteristic or this characteristic does not have the CCCD.
     */
    private boolean ensureServiceChangedEnabled(final BluetoothGatt gatt) {
        if (gatt == null)
            return false;

        // The Service Changed indications have sense only on bonded devices
        final BluetoothDevice device = gatt.getDevice();
        if (device.getBondState() != BluetoothDevice.BOND_BONDED)
            return false;

        final BluetoothGattService gaService = gatt.getService(GENERIC_ATTRIBUTE_SERVICE);
        if (gaService == null)
            return false;

        final BluetoothGattCharacteristic scCharacteristic = gaService.getCharacteristic(SERVICE_CHANGED_CHARACTERISTIC);
        if (scCharacteristic == null)
            return false;

        return enableIndications(scCharacteristic);
    }


    public final boolean enableNotifications(final BluetoothGattCharacteristic characteristic) {
        final BluetoothGatt gatt = mBluetoothGatt;
        if (gatt == null || characteristic == null)
            return false;

        // Check characteristic property
        final int properties = characteristic.getProperties();
        if ((properties & BluetoothGattCharacteristic.PROPERTY_NOTIFY) == 0)
            return false;

        gatt.setCharacteristicNotification(characteristic, true);
        final BluetoothGattDescriptor descriptor = characteristic.getDescriptor(CCCD);
        if (descriptor != null) {
            descriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
            return gatt.writeDescriptor(descriptor);
        }
        return false;
    }

    private BluetoothGattCharacteristic mNotifyCharacteristic = null;


    public boolean readDevInfo(BluetoothGattCharacteristic characteristic) {
        Log.e(TAG, "readDevInfo");
        if (mBluetoothGatt == null) {
            return false;
        }
        final int charaProp = characteristic.getProperties();
        if ((charaProp | BluetoothGattCharacteristic.PROPERTY_READ) > 0) {
            Log.e(TAG, "READ");
            // If there is an active notification on a characteristic, clear
            // it first so it doesn't update the data field on the user interface.
            if (mNotifyCharacteristic != null) {
                setCharacteristicNotification(mNotifyCharacteristic, false);
                mNotifyCharacteristic = null;
            }
            readCharacteristic(characteristic);
        }

        if ((charaProp & BluetoothGattCharacteristic.PROPERTY_WRITE) > 0) {
            if (mNotifyCharacteristic != null) {
                setCharacteristicNotification(
                        mNotifyCharacteristic, false);
                mNotifyCharacteristic = null;
            }
        }
        if ((charaProp | BluetoothGattCharacteristic.PROPERTY_NOTIFY) > 0) {
            mNotifyCharacteristic = characteristic;
            setCharacteristicNotification(characteristic, true);
        }
        return true;
    }

    public void setCharacteristicNotification(BluetoothGattCharacteristic characteristic,
                                              boolean enabled) {
        if (mBluetoothAdapter == null || mBluetoothGatt == null) {
            Log.e(TAG, "BluetoothAdapter not initialized");
            return;
        }
        mBluetoothGatt.setCharacteristicNotification(characteristic, enabled);

    }

    public final boolean enableIndications(final BluetoothGattCharacteristic characteristic) {
        final BluetoothGatt gatt = mBluetoothGatt;
        if (gatt == null || characteristic == null)
            return false;

        // Check characteristic property
        final int properties = characteristic.getProperties();
        if ((properties & BluetoothGattCharacteristic.PROPERTY_INDICATE) == 0)
            return false;

        gatt.setCharacteristicNotification(characteristic, true);
        final BluetoothGattDescriptor descriptor = characteristic.getDescriptor(CCCD);
        if (descriptor != null) {
            descriptor.setValue(BluetoothGattDescriptor.ENABLE_INDICATION_VALUE);
            return gatt.writeDescriptor(descriptor);
        }
        return false;
    }


    public final boolean readCharacteristic(final BluetoothGattCharacteristic characteristic) {
        final BluetoothGatt gatt = mBluetoothGatt;
        if (gatt == null || characteristic == null)
            return false;

        // Check characteristic property
        final int properties = characteristic.getProperties();
        if ((properties & BluetoothGattCharacteristic.PROPERTY_READ) == 0)
            return false;

        return gatt.readCharacteristic(characteristic);
    }

    public final boolean writeCharacteristic(final BluetoothGattCharacteristic characteristic) {
        final BluetoothGatt gatt = mBluetoothGatt;
        if (gatt == null || characteristic == null)
            return false;

        // Check characteristic property
        final int properties = characteristic.getProperties();
        return !((properties & (BluetoothGattCharacteristic.PROPERTY_WRITE | BluetoothGattCharacteristic.PROPERTY_WRITE_NO_RESPONSE)) == 0) && gatt.writeCharacteristic(characteristic);
    }

    public BluetoothGattCharacteristic getCharacteristicFromService(UUID service, UUID characteristic) {
        if (mBluetoothGatt != null) {
            BluetoothGattService gattService = mBluetoothGatt.getService(service);
            if (gattService != null) {
                return gattService.getCharacteristic(characteristic);
            }
        }
        return null;
    }

    public List<BluetoothGattService> getBluetoothGattServices() {
        if (mBluetoothGatt != null)
            return mBluetoothGatt.getServices();
        else
            return null;
    }

    public int getBluetoothGattDescriptor(BluetoothGattCharacteristic characteristic) {
        int i = 0;
        if (characteristic != null)
            i = characteristic.getProperties();
        return i;
    }

    public List<BluetoothGattCharacteristic> getBluetoothGattCharacteristics(BluetoothGattService service) {
        if (mBluetoothGatt != null && service != null) {
            return service.getCharacteristics();
        }
        return null;
    }

    public boolean isFoundService() {
        if (mBluetoothGatt != null) {
            BluetoothGattService service = mBluetoothGatt.getService(RX_SERVICE_UUID);
            if (service != null) {
                if (service.getCharacteristic(RX_CHAR_UUID) != null && service.getCharacteristic(TX_CHAR_UUID) != null)
                    return true;
            }
        }
        return false;
    }

    public boolean send(byte[] s) {
        if (mBluetoothGatt != null) {
            BluetoothGattService service = mBluetoothGatt.getService(RX_SERVICE_UUID);
            if (service != null) {
                BluetoothGattCharacteristic characteristic = service.getCharacteristic(TX_CHAR_UUID);
                if (characteristic != null) {
                    characteristic.setValue(s);
                    return mBluetoothGatt.writeCharacteristic(characteristic);
                }
            }
        }
        return false;
    }


    public static final class Request {
        private enum Type {
            WRITE,
            READ,
            ENABLE_NOTIFICATIONS,
            ENABLE_INDICATIONS
        }

        private final Type type;
        private final BluetoothGattCharacteristic characteristic;
        private final byte[] value;

        private Request(final Type type, final BluetoothGattCharacteristic characteristic) {
            this.type = type;
            this.characteristic = characteristic;
            this.value = null;
        }

        private Request(final Type type, final BluetoothGattCharacteristic characteristic, final byte[] value) {
            this.type = type;
            this.characteristic = characteristic;
            this.value = value;
        }

        public static Request newReadRequest(final BluetoothGattCharacteristic characteristic) {
            return new Request(Type.READ, characteristic);
        }

        public static Request newWriteRequest(final BluetoothGattCharacteristic characteristic, final byte[] value) {
            return new Request(Type.WRITE, characteristic, value);
        }

        public static Request newEnableNotificationsRequest(final BluetoothGattCharacteristic characteristic) {
            return new Request(Type.ENABLE_NOTIFICATIONS, characteristic);
        }

        public static Request newEnableIndicationsRequest(final BluetoothGattCharacteristic characteristic) {
            return new Request(Type.ENABLE_INDICATIONS, characteristic);
        }
    }

//    public  boolean isfoundService(BluetoothGatt gatt){
//        return mBleCallBack.isFoundService(gatt);
//    }

    public BluetoothGattCallback mBluetoothGattCallback = new BluetoothGattCallback() {

        @Override
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            super.onMtuChanged(gatt, mtu, status);
        }

        @Override
        public void onReadRemoteRssi(BluetoothGatt gatt, int rssi, int status) {
            super.onReadRemoteRssi(gatt, rssi, status);

            if (status == BluetoothGatt.GATT_SUCCESS) {
                mBleCallBack.onReadRssi(rssi);
            } else {
                mBleCallBack.onError(Constants.ERROR_READRSSI, "onReadRemoteRssi" + GattError.parse(status));
            }
        }

        @Override
        public void onReliableWriteCompleted(BluetoothGatt gatt, int status) {
            Log.e(TAG, "send a data" + status);
            mBleCallBack.onDataSendOver();
            super.onReliableWriteCompleted(gatt, status);
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorWrite(gatt, descriptor, status);
        }

        @Override
        public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorRead(gatt, descriptor, status);
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            Log.e(TAG, "receive a data" + characteristic.getValue());
            mBleCallBack.onReadData(gatt, characteristic);
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            Log.e(TAG, "send a data" + characteristic.getValue());
            super.onCharacteristicWrite(gatt, characteristic, status);
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            super.onCharacteristicRead(gatt, characteristic, status);
            Log.e(TAG, "receive a data" + characteristic.getValue());
            onReadData(gatt, characteristic, status);
        }

        @Override
        public void onServicesDiscovered(final BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                DebugLogger.e(TAG, "ondiscovered...");
//                mBleCallBack.onDiscoverServer();
                // Obtain the queue of initialization requests
                mHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        if (isFoundService()) {
                            mInitInProgress = true;
                            mQueue = initGatt();
                            // When the device is bonded and has Service Changed characteristic, the indications must be enabled first.
                            // In case this method returns true we have to continue in the onDescriptorWrite callback
                            if (ensureServiceChangedEnabled(gatt))
                                return;

                            // We have discovered services, proceed with the initialization queue.
                            nextRequest();
                        } else {
                            DebugLogger.e(TAG, "ondiscovered failed");
                            mBleCallBack.onDeviceNotSupported();
                            disconnect();
                        }
                    }
                });
            } else {
                DebugLogger.e(TAG, "onServicesDiscovered error " + status);
                mBleCallBack.onError(status, "onServicesDiscovered:" + ERROR_DISCOVERY_SERVICE);
            }
        }


        @Override
        public void onConnectionStateChange(final BluetoothGatt gatt, int status, int newState) {
            if (status == BluetoothGatt.GATT_SUCCESS && newState == BluetoothProfile.STATE_CONNECTED) {
                mSatuts = Constants.CONNECTTED;
                mHandler.removeMessages(TIMEOUT);
                mBleCallBack.onDeviceConnected();
                mHandler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        // Some proximity tags (e.g. nRF PROXIMITY) initialize bonding automatically when connected.
                        if (gatt.getDevice().getBondState() != BluetoothDevice.BOND_BONDING) {
                            gatt.discoverServices();
                        }
                    }
                }, 600);
            } else {
                if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                    mSatuts = Constants.DISCONNECT;
                    if (mUserDisconnected) {
                        mBleCallBack.onDeviceDisconnected();
                        close();
                    } else {
                        mBleCallBack.onLinklossOccur();
                        // We are not closing the connection here as the device should try to reconnect automatically.
                        // This may be only called when the shouldAutoConnect() method returned true.
                    }
                    return;
                }
                // TODO Should the disconnect method be called or the connection is still valid? Does this ever happen?
                mBleCallBack.onError(status, "onConnectionStateChange:" + ERROR_CONNECTION_STATE_CHANGE);
            }
        }

        private Queue<Request> initGatt() {
            final LinkedList<Request> requestLinkedList = new LinkedList<>();
            requestLinkedList.add(new Request(Request.Type.ENABLE_NOTIFICATIONS, getCharacteristicFromService(RX_SERVICE_UUID, RX_CHAR_UUID)));
            return requestLinkedList;
        }
    };

    public void nextRequest() {
        final Queue<Request> requests = mQueue;

        // Get the first request from the queue
        final Request request = requests != null ? requests.poll() : null;

        // Are we done?
        if (request == null) {
            if (mInitInProgress) {
                mInitInProgress = false;
                mBleCallBack.onDeviceReady();
            }
            return;
        }

        switch (request.type) {
            case READ: {
                readCharacteristic(request.characteristic);
                break;
            }
            case WRITE: {
                final BluetoothGattCharacteristic characteristic = request.characteristic;
                characteristic.setValue(request.value);
                writeCharacteristic(characteristic);
                break;
            }
            case ENABLE_NOTIFICATIONS: {
                if (enableNotifications(request.characteristic)){
                    mBleCallBack.onDeviceReady();
                }
                break;
            }
            case ENABLE_INDICATIONS: {
                enableIndications(request.characteristic);
                break;
            }
        }
    }

}
